@@ -3,11 +3,13 @@ A module exposing data-file encoders and decoders.
 """
 
 # built-in
+from contextlib import contextmanager as _contextmanager
 from enum import Enum
 import logging
 from pathlib import Path
 from typing import Any as _Any
 from typing import Callable as _Callable
+from typing import Iterator as _Iterator
 from typing import NamedTuple
 from typing import Optional as _Optional
 
@@ -31,7 +33,9 @@ from vcorelib.io.types import DataDecoder as _DataDecoder
 from vcorelib.io.types import DataEncoder as _DataEncoder
 from vcorelib.io.types import DataStream as _DataStream
 from vcorelib.io.types import EncodeResult as _EncodeResult
-from vcorelib.io.types import FileExtension, LoadResult
+from vcorelib.io.types import FileExtension
+from vcorelib.io.types import JsonObject as _JsonObject
+from vcorelib.io.types import LoadResult
 from vcorelib.io.types import StreamProcessor as _StreamProcessor
 from vcorelib.paths import Pathlike as _Pathlike
 from vcorelib.paths import find_file, get_file_ext, get_file_name, normalize
@@ -311,5 +315,50 @@ class DataArbiter:
             self.logger.warning("No encoder for '%s'.", ext)
         return result
 
+    @_contextmanager
+    def object_file_context(
+        self,
+        pathlike: _Pathlike,
+        decode_kwargs: dict = None,
+        encode_kwargs: dict = None,
+    ) -> _Iterator[_JsonObject]:
+        """
+        Provide data loaded from a file as a context so that it's written back
+        when the context ends.
+        """
+        if decode_kwargs is None:
+            decode_kwargs = {}
+        if encode_kwargs is None:
+            encode_kwargs = {}
+        data = self.decode(pathlike, **decode_kwargs).data
+        yield data
+        self.encode(pathlike, data, **encode_kwargs)
+
+    @_contextmanager
+    def object_directory_context(
+        self,
+        pathlike: _Pathlike,
+        decode_kwargs: dict = None,
+        encode_kwargs: dict = None,
+    ) -> _Iterator[_JsonObject]:
+        """Provide a loaded directory as a context."""
+
+        if decode_kwargs is None:
+            decode_kwargs = {}
+        if encode_kwargs is None:
+            encode_kwargs = {}
+
+        # Make sure the directory exists, since we would create it anyway.
+        path = normalize(pathlike)
+        path.mkdir(parents=True, exist_ok=True)
+
+        data = self.decode_directory(path, **decode_kwargs).data
+        yield data
+        self.encode_directory(
+            pathlike,
+            data,
+            **encode_kwargs,
+        )
+
 
 ARBITER = DataArbiter()
